/* Copyright Rudolf Koenig, 2008.
   Released under the GPL Licence, Version 2
   Inpired by the MyUSB USBtoSerial demo, Copyright (C) Dean Camera, 2008.

   Modified by Stefan Strmberg 2008 for NetHomeServer
*/

#include <avr/io.h>
#include <avr/boot.h>
#include <avr/eeprom.h>
#include <string.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>

#include <MyUSB/Drivers/USB/USB.h>     // USB Functionality
#include <MyUSB/Scheduler/Scheduler.h> // Simple scheduler for task management

#include "spi.h"
#include "cc1100.h"
#include "cdc.h"
#include "clock.h"
#include "delay.h"
#include "display.h"
#include "fncollection.h"
#include "led.h"
#include "ringbuffer.h"
#include "raw_transceiver.h"
#include "ttydata.h"
#include "fht.h"


/* Scheduler Task List */
TASK_LIST
{
  { Task: USB_USBTask  , TaskStatus: TASK_STOP },
  { Task: CDC_Task     , TaskStatus: TASK_STOP },
  { Task: ReportTask   , TaskStatus: TASK_RUN },
  { Task: Minute_Task,   TaskStatus: TASK_RUN },
};

t_fntab fntab[] = {

  { 'A', addRawPulse },
  { 'B', prepare_boot },
  { 'C', ccreg },
  { 'E', resetSendBuffer },
  { 'I', readConfiguration },
  { 'O', writeConfiguration },
  { 'R', read_eeprom },
  { 'S', sendRawMessage },
  { 'V', version },
  { 'W', write_eeprom },
  { 'X', setReportState },

  { 'e', eeprom_factory_reset },
  { 'l', ledfunc },
  { 't', gettime },
  { 'x', ccsetpa },

  { 0, 0 },
};

void
start_bootloader(void)
{
  cli();

  /* move interrupt vectors to bootloader section and jump to bootloader */
  MCUCR = _BV(IVCE);
  MCUCR = _BV(IVSEL);

#ifdef CUL_V3
#  define jump_to_bootloader ((void(*)(void))0x3800)
#else
#  define jump_to_bootloader ((void(*)(void))0x1800)
#endif
  jump_to_bootloader();
}

EVENT_HANDLER(USB_Connect)
{
  Scheduler_SetTaskMode(USB_USBTask, TASK_RUN);
}

EVENT_HANDLER(USB_Disconnect)
{
  Scheduler_SetTaskMode(USB_USBTask, TASK_STOP);
  Scheduler_SetTaskMode(CDC_Task,    TASK_STOP);
  ccStrobe(CC1100_SIDLE);
}

int
main(void)
{

  PORTB |= _BV( PB6 ); // Pull 433MHz marker

  spi_init();
  eeprom_init();

  // if we had been restarted by watchdog check the REQ BootLoader byte in the
  // EEPROM ...
  if(bit_is_set(MCUSR,WDRF) && eeprom_read_byte(EE_REQBL)) {
    eeprom_write_byte( EE_REQBL, 0 ); // clear flag
    start_bootloader();
  }

  // Setup the timers. Are needed for watchdog-reset
  OCR0A  = 249;                            // Timer0: 0.008s = 8MHz/256/250
  TCCR0B = _BV(CS02);
  TCCR0A = _BV(WGM01);
  TIMSK0 = _BV(OCIE0A);

  TCCR1A = 0;
  TCCR1B = _BV(CS11) | _BV(WGM12);         // Timer1: 1us = 8MHz/8

  SetSystemClockPrescaler(0);              // Disable Clock Division

  MCUSR &= ~(1 << WDRF);                   // Enable the watchdog
  wdt_enable(WDTO_2S);

  led_init();
  rb_init(USB_Tx_Buffer, CDC_TX_EPSIZE);
  rb_init(USB_Rx_Buffer, CDC_RX_EPSIZE);
  Scheduler_Init();
  USB_Init();
  tty_init();
  ccInitChip();
  //fht_init();
  LED_OFF();

  credit_10ms = MAX_CREDIT/2;
  output_enabled = OUTPUT_USB;

  Scheduler_Start();                       // Won't return
}
